from pathlib import Path

import click


@click.command(name="example", help="Print config or arg example for a module and exit")
@click.argument("module", type=click.Choice(["balance", "transfer-sol"]))
def cli(module: str) -> None:
    example_file = Path(Path(__file__).parent.absolute(), "../examples", f"{module}.yml")
    click.echo(example_file.read_text())



import os

import pytest
from dotenv import load_dotenv

load_dotenv(".env")


@pytest.fixture
def mainnet_node():
    return os.getenv("MAINNET_NODE")


@pytest.fixture
def testnet_node():
    return os.getenv("TESTNET_NODE")


@pytest.fixture
def usdt_token_address():
    return os.getenv("USDT_TOKEN_ADDRESS")


@pytest.fixture
def usdt_owner_address():
    return os.getenv("USDT_OWNER_ADDRESS")


@pytest.fixture
def binance_wallet():
    return "2ojv9BAiHUrvsm9gxDe7fJSzbNZSJcxZvf8dqmWGHG8S"



import random
from decimal import Decimal

import click
from click import Context
from mm_std import print_console, str_to_list
from pydantic import StrictStr, field_validator

from mm_solana.cli.helpers import BaseCmdConfig, parse_config, print_config_and_exit
from mm_solana.transfer import transfer_sol


class Config(BaseCmdConfig):
    from_address: StrictStr
    private_key: StrictStr
    recipients: list[StrictStr]
    nodes: list[StrictStr]
    amount: Decimal

    @field_validator("recipients", "nodes", mode="before")
    def to_list_validator(cls, v: list[str] | str | None) -> list[str]:
        return str_to_list(v)

    @property
    def random_node(self) -> str:
        return random.choice(self.nodes)


@click.command(name="transfer-sol", help="Transfer SOL")
@click.argument("config_path", type=click.Path(exists=True))
@click.pass_context
def cli(ctx: Context, config_path: str) -> None:
    config = parse_config(ctx, config_path, Config)
    print_config_and_exit(ctx, config)
    result = {}
    for recipient in config.recipients:
        res = transfer_sol(
            from_address=config.from_address,
            private_key_base58=config.private_key,
            recipient_address=recipient,
            amount_sol=config.amount,
            nodes=config.nodes,
        )
        result[recipient] = res.ok_or_err()
    print_console(result, print_json=True)



import random
from decimal import Decimal
from typing import Any

import click
from click import Context
from mm_std import print_console, str_to_list
from pydantic import StrictStr, field_validator

from mm_solana import balance, utils
from mm_solana.cli.helpers import BaseCmdConfig, parse_config, print_config_and_exit


class Config(BaseCmdConfig):
    accounts: list[StrictStr]
    nodes: list[StrictStr]
    tokens: list[StrictStr] | None = None

    @field_validator("accounts", "nodes", "tokens", mode="before")
    def to_list_validator(cls, v: list[str] | str | None) -> list[str]:
        return str_to_list(v)

    @property
    def random_node(self) -> str:
        return random.choice(self.nodes)


@click.command(name="balance", help="Print SOL and tokens balances")
@click.argument("config_path", type=click.Path(exists=True))
@click.pass_context
def cli(ctx: Context, config_path: str) -> None:
    config = parse_config(ctx, config_path, Config)
    print_config_and_exit(ctx, config)
    result: dict[str, Any] = {"sol": _get_sol_balances(config.accounts, config.nodes)}
    result["sol_sum"] = sum([v for v in result["sol"].values() if v is not None])

    if config.tokens:
        for token in config.tokens:
            result[token] = _get_token_balances(token, config.accounts, config.nodes)
            result[token + "_sum"] = sum([v for v in result[token].values() if v is not None])

    print_console(result, print_json=True)


def _get_token_balances(token: str, accounts: list[str], nodes: list[str]) -> dict[str, int | None]:
    result = {}
    for account in accounts:
        # result[account] = _get_token_balance(token, account, nodes)
        result[account] = balance.token_balance(
            token_mint_address=token,
            wallet_address=account,
            nodes=nodes,
            attempts=3,
        ).ok_or_none()
    return result


def _get_sol_balances(accounts: list[str], nodes: list[str]) -> dict[str, Decimal | None]:
    result = {}
    for account in accounts:
        res = balance.sol_balance(address=account, nodes=nodes)
        result[account] = utils.lamports_to_sol(res.unwrap(), ndigits=2) if res.is_ok() else None
    return result



import click
from click import Context
from click_aliases import ClickAliasedGroup

from mm_solana.cli.cmd import (
    balance_cmd,
    example_cmd,
    generate_accounts_cmd,
    keypair_cmd,
    transfer_sol_cmd,
)
from mm_solana.cli.helpers import get_version


@click.group(cls=ClickAliasedGroup)
@click.option("-c", "--config/--no-config", "config_", default=False, help="Print config and exit")
@click.option("-n", "--node", multiple=True, help="List of JSON RPC nodes, it overwrites node/nodes field in config")
@click.version_option(get_version(), help="Show the version and exit")
@click.help_option(help="Show this message and exit")
@click.pass_context
def cli(ctx: Context, config_: bool, node: list[str]) -> None:
    ctx.ensure_object(dict)
    ctx.obj["config"] = config_
    ctx.obj["nodes"] = node


# noinspection PyTypeChecker
cli.add_command(balance_cmd.cli)
# noinspection PyTypeChecker
cli.add_command(example_cmd.cli)
# noinspection PyTypeChecker
cli.add_command(generate_accounts_cmd.cli)
# noinspection PyTypeChecker
cli.add_command(keypair_cmd.cli)
# noinspection PyTypeChecker
cli.add_command(transfer_sol_cmd.cli)



from mm_solana import rpc


def test_get_balance(mainnet_node, binance_wallet):
    res = rpc.get_balance(mainnet_node, binance_wallet)
    assert res.unwrap() > 10_000_000


def test_get_slot(testnet_node):
    res = rpc.get_slot(testnet_node)
    assert res.unwrap() > 10_000


def test_get_epoch_info(testnet_node):
    res = rpc.get_epoch_info(testnet_node)
    assert res.unwrap().epoch > 500


def test_get_health(mainnet_node, testnet_node):
    res = rpc.get_health(mainnet_node)
    assert res.unwrap() is True

    res = rpc.get_health(testnet_node)
    assert res.unwrap() is True


def test_get_cluster_nodes(testnet_node):
    res = rpc.get_cluster_nodes(testnet_node)
    assert len(res.unwrap()) > 10


def test_get_vote_accounts(testnet_node):
    res = rpc.get_vote_accounts(testnet_node, timeout=60)
    assert len(res.unwrap()) > 10


def test_get_leader_scheduler(testnet_node):
    res = rpc.get_leader_scheduler(testnet_node)
    assert len(res.unwrap()) > 10


def test_get_block_production(testnet_node):
    res = rpc.get_block_production(testnet_node)
    assert res.unwrap().slot > 1000


def test_get_stake_activation(mainnet_node):
    res = rpc.get_stake_activation(mainnet_node, "GitYucwpNcg6Dx1Y15UQ9TQn8LZMX1uuqQNn8rXxEWNC")  # from kraken
    assert res.unwrap().state == "active"


def test_(mainnet_node):
    tx_hash = "2vifJ5g4inS4spZLQMUyVstvMrCM2mg1QC9xjD6bgsiMUwp8sTE5waCdshJ8SVaH95WGtexjH3q8ot1GoKe9yK3h"
    res = rpc.get_transaction(mainnet_node, tx_hash, 0)
    assert res.unwrap()["blockTime"] == 1708667439
